@@ -309,6 +309,36 @@ gpk_update_enum_to_text (GpkUpdateEnum update)
 	return pk_enum_find_string (enum_update, update);
 }
 
+/**
+ * gpk_media_type_enum_to_localised_text:
+ **/
+const gchar *
+gpk_media_type_enum_to_localised_text (PkMediaTypeEnum type)
+{
+	const gchar *text = NULL;
+	switch (type) {
+	case PK_MEDIA_TYPE_ENUM_CD:
+		/* TRANSLATORS: this is compact disk (CD) media */
+		text = _("CD");
+		break;
+	case PK_MEDIA_TYPE_ENUM_DVD:
+		/* TRANSLATORS: this is digital versatile disk (DVD) media */
+		text = _("DVD");
+		break;
+	case PK_MEDIA_TYPE_ENUM_DISC:
+		/* TRANSLATORS: this is either CD or DVD media */
+		text = _("disc");
+		break;
+	case PK_MEDIA_TYPE_ENUM_UNKNOWN:
+		/* TRANSLATORS: this is generic media of unknown type that we will install from */
+		text = _("media");
+		break;
+	default:
+		egg_warning ("Unknown media type");
+	}
+	return text;
+}
+
 /**
  * gpk_error_enum_to_localised_text:
  **/
@@ -455,6 +485,12 @@ gpk_error_enum_to_localised_text (PkErrorCodeEnum code)
 	case PK_ERROR_ENUM_NO_SPACE_ON_DEVICE:
 		text = _("No space is left on the disk");
 		break;
+	case PK_ERROR_ENUM_MEDIA_CHANGE_REQUIRED:
+		text = _("A media change is required");
+		break;
+	case PK_ERROR_ENUM_NOT_AUTHORIZED:
+		text = _("Authorization failed");
+		break;
 	default:
 		egg_warning ("Unknown error");
 	}
@@ -638,6 +674,12 @@ gpk_error_enum_to_localised_message (PkErrorCodeEnum code)
 		text = _("There is insufficient space on the device.\n"
 			 "Free some space on the system disk to perform this operation.");
 		break;
+	case PK_ERROR_ENUM_MEDIA_CHANGE_REQUIRED:
+		text = _("Additional media is required to complete the transaction.");
+		break;
+	case PK_ERROR_ENUM_NOT_AUTHORIZED:
+		text = _("You have failed to provide correct authentication. Please check any passwords or account settings.");
+		break;
 	default:
 		egg_warning ("Unknown error, please report a bug at " GPK_BUGZILLA_URL ".\n"
 			    "More information is available in the detailed report.");
@@ -686,7 +728,7 @@ gpk_restart_enum_to_localised_text (PkRestartEnum restart)
 		text = _("A restart is required");
 		break;
 	case PK_RESTART_ENUM_SESSION:
-		text = _("You will need to log out and log back in");
+		text = _("You need to log out and log back in");
 		break;
 	case PK_RESTART_ENUM_APPLICATION:
 		text = _("You need to restart the application");
@@ -824,7 +866,7 @@ gpk_status_enum_to_localised_text (PkStatusEnum status)
 		break;
 	case PK_STATUS_ENUM_UPDATE:
 		/* TRANSLATORS: The transaction state */
-		text = _("Updating packages");
+		text = _("Installing updates");
 		break;
 	case PK_STATUS_ENUM_CLEANUP:
 		/* TRANSLATORS: The transaction state */
@@ -1590,6 +1632,38 @@ gpk_message_enum_to_icon_name (PkMessageEnum message)
 	return pk_enum_find_string (enum_message_icon_name, message);
 }
 
+/**
+ * gpk_info_status_enum_to_text:
+ **/
+const gchar *
+gpk_info_status_enum_to_text (GpkInfoStatusEnum info)
+{
+	if (info > PK_INFO_ENUM_UNKNOWN)
+		return gpk_info_enum_to_localised_past (info - PK_INFO_ENUM_UNKNOWN);
+	return gpk_info_enum_to_localised_present (info);
+}
+
+/**
+ * gpk_info_status_enum_to_icon_name:
+ **/
+const gchar *
+gpk_info_status_enum_to_icon_name (GpkInfoStatusEnum info)
+{
+	/* special hardcoded icons */
+	if (info == GPK_INFO_ENUM_DOWNLOADED)
+		return "pk-package-downloaded";
+	if (info == GPK_INFO_ENUM_INSTALLED ||
+	    info == GPK_INFO_ENUM_UPDATED)
+		return "pk-package-installed";
+
+	/* use normal icon as a fallback */
+	if (info > PK_INFO_ENUM_UNKNOWN)
+		return gpk_info_enum_to_icon_name (info - PK_INFO_ENUM_UNKNOWN);
+
+	/* regular PkInfoEnum */
+	return gpk_info_enum_to_icon_name (info);
+}
+
 /***************************************************************************
  ***                          MAKE CHECK TESTS                           ***
  ***************************************************************************/
