@@ -52,6 +52,48 @@ typedef enum {
 	GPK_UPDATE_ENUM_UNKNOWN
 } GpkUpdateEnum;
 
+typedef enum {
+	GPK_INFO_ENUM_DOWNLOADING	= PK_INFO_ENUM_DOWNLOADING,
+	GPK_INFO_ENUM_UPDATING		= PK_INFO_ENUM_UPDATING,
+	GPK_INFO_ENUM_INSTALLING	= PK_INFO_ENUM_INSTALLING,
+	GPK_INFO_ENUM_REMOVING		= PK_INFO_ENUM_REMOVING,
+	GPK_INFO_ENUM_CLEANUP		= PK_INFO_ENUM_CLEANUP,
+	GPK_INFO_ENUM_OBSOLETING	= PK_INFO_ENUM_OBSOLETING,
+	GPK_INFO_ENUM_DOWNLOADED	= PK_INFO_ENUM_UNKNOWN + PK_INFO_ENUM_DOWNLOADING,
+	GPK_INFO_ENUM_UPDATED		= PK_INFO_ENUM_UNKNOWN + PK_INFO_ENUM_UPDATING,
+	GPK_INFO_ENUM_INSTALLED		= PK_INFO_ENUM_UNKNOWN + PK_INFO_ENUM_INSTALLING,
+	GPK_INFO_ENUM_REMOVED		= PK_INFO_ENUM_UNKNOWN + PK_INFO_ENUM_REMOVING,
+	GPK_INFO_ENUM_CLEANEDUP		= PK_INFO_ENUM_UNKNOWN + PK_INFO_ENUM_CLEANUP,
+	GPK_INFO_ENUM_OBSOLETED		= PK_INFO_ENUM_UNKNOWN + PK_INFO_ENUM_OBSOLETING,
+	GPK_INFO_ENUM_UNKNOWN
+} GpkInfoStatusEnum;
+
+/* for very old versions of PackageKit */
+#ifndef PK_CHECK_VERSION
+#define PK_CHECK_VERSION(major, minor, micro) 0
+#endif
+
+/* constants defined in 0.4.5 */
+#if (!PK_CHECK_VERSION(0,4,5))
+#define PK_INFO_ENUM_FINISHED			(PK_INFO_ENUM_COLLECTION_AVAILABLE + 1)
+#endif
+
+/* constants defined in 0.4.7 */
+#if (!PK_CHECK_VERSION(0,4,7))
+typedef guint PkMediaTypeEnum;
+#define PK_MEDIA_TYPE_ENUM_CD			(0)
+#define PK_MEDIA_TYPE_ENUM_DVD			(1)
+#define PK_MEDIA_TYPE_ENUM_DISC			(2)
+#define PK_MEDIA_TYPE_ENUM_UNKNOWN		(3)
+#define PK_EXIT_ENUM_MEDIA_CHANGE_REQUIRED	(PK_EXIT_ENUM_KILLED + 1)
+#define PK_ERROR_ENUM_MEDIA_CHANGE_REQUIRED	(PK_ERROR_ENUM_NO_SPACE_ON_DEVICE + 1)
+#endif
+
+/* constants defined in 0.4.8 */
+#if (!PK_CHECK_VERSION(0,4,8))
+#define PK_ERROR_ENUM_NOT_AUTHORIZED		(PK_ERROR_ENUM_MEDIA_CHANGE_REQUIRED + 1)
+#endif
+
 void		 gpk_enum_test				(gpointer	 data);
 const gchar	*gpk_role_enum_to_localised_past	(PkRoleEnum	 role)
 							 G_GNUC_CONST;
@@ -62,6 +104,8 @@ const gchar	*gpk_freq_enum_to_text			(GpkFreqEnum	 freq);
 GpkUpdateEnum	 gpk_update_enum_from_text		(const gchar	*update);
 const gchar	*gpk_update_enum_to_text		(GpkUpdateEnum	 update);
 const gchar	*gpk_role_enum_to_icon_name		(PkRoleEnum	 role);
+const gchar	*gpk_media_type_enum_to_localised_text	(PkMediaTypeEnum type)
+							 G_GNUC_CONST;
 const gchar	*gpk_info_enum_to_localised_text	(PkInfoEnum	 info)
 							 G_GNUC_CONST;
 const gchar	*gpk_info_enum_to_localised_past	(PkInfoEnum	 info)
@@ -93,6 +137,8 @@ const gchar	*gpk_group_enum_to_icon_name		(PkGroupEnum	 group);
 gchar		*gpk_update_enum_to_localised_text	(PkInfoEnum	 info,
 							 guint		 number)
 							 G_GNUC_CONST;
+const gchar	*gpk_info_status_enum_to_text		(GpkInfoStatusEnum info);
+const gchar	*gpk_info_status_enum_to_icon_name	(GpkInfoStatusEnum info);
 
 G_END_DECLS
 
