@@ -35,7 +35,6 @@
 #include <glib/gi18n.h>
 
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <gconf/gconf-client.h>
 #include <libnotify/notify.h>
 #include <polkit-gnome/polkit-gnome.h>
@@ -47,9 +46,8 @@
 #include "gpk-common.h"
 #include "gpk-error.h"
 #include "gpk-watch.h"
-#include "gpk-client.h"
+#include "gpk-modal-dialog.h"
 #include "gpk-inhibit.h"
-#include "gpk-smart-icon.h"
 #include "gpk-consolekit.h"
 #include "gpk-enum.h"
 
@@ -64,18 +62,17 @@ static void     gpk_watch_finalize	(GObject       *object);
 struct GpkWatchPrivate
 {
 	PkControl		*control;
-	GpkSmartIcon		*sicon;
+	GtkStatusIcon		*status_icon;
 	GPtrArray		*cached_messages;
 	GPtrArray		*restart_package_names;
 	NotifyNotification	*notification_cached_messages;
 	GpkInhibit		*inhibit;
-	GpkClient		*gclient;
-	GpkClient		*monitor;
+	GpkModalDialog		*dialog;
+	PkClient		*client_primary;
 	PkConnection		*pconnection;
 	PkTaskList		*tlist;
 	PkRestartEnum		 restart;
 	GConfClient		*gconf_client;
-	gboolean		 show_refresh_in_menu;
 	PolKitGnomeAction	*restart_action;
 	guint			 set_proxy_timeout;
 	gchar			*error_details;
@@ -214,9 +211,9 @@ gpk_watch_refresh_tooltip (GpkWatch *watch)
 
 out:
 #if GTK_CHECK_VERSION(2,15,0)
-	gtk_status_icon_set_tooltip_text (GTK_STATUS_ICON (watch->priv->sicon), status->str);
+	gtk_status_icon_set_tooltip_text (watch->priv->status_icon, status->str);
 #else
-	gtk_status_icon_set_tooltip (GTK_STATUS_ICON (watch->priv->sicon), status->str);
+	gtk_status_icon_set_tooltip (watch->priv->status_icon, status->str);
 #endif
 	g_string_free (status, TRUE);
 	return TRUE;
@@ -228,6 +225,8 @@ out:
 static PkBitfield
 gpk_watch_task_list_to_status_bitfield (GpkWatch *watch)
 {
+	gboolean ret;
+	gboolean active;
 	guint i;
 	guint length;
 	PkBitfield status = 0;
@@ -247,8 +246,18 @@ gpk_watch_task_list_to_status_bitfield (GpkWatch *watch)
 			egg_warning ("not found item %i", i);
 			break;
 		}
-		egg_debug ("%s %s", item->tid, pk_status_enum_to_text (item->status));
-		pk_bitfield_add (status, item->status);
+
+		/* only show an icon for this if the application isn't still on the bus */
+		ret = pk_client_is_caller_active (item->monitor, &active, NULL);
+
+		/* if we failed to get data, assume bad things happened */
+		if (!ret)
+			active = TRUE;
+
+		/* add to bitfield calculation */
+		egg_debug ("%s %s (active:%i)", item->tid, pk_status_enum_to_text (item->status), active);
+		if (!active)
+			pk_bitfield_add (status, item->status);
 	}
 out:
 	return status;
@@ -328,7 +337,13 @@ gpk_watch_refresh_icon (GpkWatch *watch)
 	}
 
 out:
-	gpk_smart_icon_set_icon_name (watch->priv->sicon, icon_name);
+	/* no icon, hide */
+	if (icon_name == NULL) {
+		gtk_status_icon_set_visible (watch->priv->status_icon, FALSE);
+		return FALSE;
+	}
+	gtk_status_icon_set_from_icon_name (watch->priv->status_icon, icon_name);
+	gtk_status_icon_set_visible (watch->priv->status_icon, TRUE);
 	return TRUE;
 }
 
@@ -340,13 +355,6 @@ gpk_watch_task_list_changed_cb (PkTaskList *tlist, GpkWatch *watch)
 {
 	g_return_if_fail (GPK_IS_WATCH (watch));
 
-	if (pk_task_list_contains_role (tlist, PK_ROLE_ENUM_REFRESH_CACHE) ||
-	    pk_task_list_contains_role (tlist, PK_ROLE_ENUM_UPDATE_PACKAGES) ||
-	    pk_task_list_contains_role (tlist, PK_ROLE_ENUM_UPDATE_SYSTEM))
-		watch->priv->show_refresh_in_menu = FALSE;
-	else
-		watch->priv->show_refresh_in_menu = TRUE;
-
 	gpk_watch_refresh_icon (watch);
 	gpk_watch_refresh_tooltip (watch);
 }
@@ -373,10 +381,10 @@ gpk_watch_libnotify_cb (NotifyNotification *notification, gchar *action, gpointe
 }
 
 /**
- * gpk_watch_finished_cb:
+ * gpk_watch_task_list_finished_cb:
  **/
 static void
-gpk_watch_finished_cb (PkTaskList *tlist, PkClient *client, PkExitEnum exit_enum, guint runtime, GpkWatch *watch)
+gpk_watch_task_list_finished_cb (PkTaskList *tlist, PkClient *client, PkExitEnum exit_enum, guint runtime, GpkWatch *watch)
 {
 	guint i;
 	gboolean ret;
@@ -641,7 +649,7 @@ gpk_watch_message_cb (PkTaskList *tlist, PkClient *client, PkMessageEnum message
 	}
 
 	/* do the bubble */
-	notification = notify_notification_new_with_status_icon (_("New package manager message"), NULL, "emblem-important", GTK_STATUS_ICON(watch->priv->sicon));
+	notification = notify_notification_new_with_status_icon (_("New package manager message"), NULL, "emblem-important", watch->priv->status_icon);
 	notify_notification_set_timeout (notification, NOTIFY_EXPIRES_NEVER);
 	notify_notification_set_urgency (notification, NOTIFY_URGENCY_LOW);
 	ret = notify_notification_show (notification, &error);
@@ -793,27 +801,6 @@ gpk_watch_restart_cb (PolKitGnomeAction *action, gpointer data)
 	gpk_restart_system ();
 }
 
-/**
- * gpk_watch_menu_refresh_cache_cb:
- **/
-static void
-gpk_watch_menu_refresh_cache_cb (GtkMenuItem *item, gpointer data)
-{
-	gboolean ret;
-	GpkWatch *watch = GPK_WATCH (data);
-	GError *error = NULL;
-
-	g_return_if_fail (GPK_IS_WATCH (watch));
-
-	egg_debug ("refresh cache");
-	gpk_client_set_interaction (watch->priv->gclient, GPK_CLIENT_INTERACT_ALWAYS);
-	ret = gpk_client_refresh_cache (watch->priv->gclient, &error);
-	if (!ret) {
-		egg_warning ("%s", error->message);
-		g_error_free (error);
-	}
-}
-
 /**
  * gpk_watch_menu_show_messages_cb:
  **/
@@ -821,7 +808,7 @@ static void
 gpk_watch_menu_show_messages_cb (GtkMenuItem *item, gpointer data)
 {
 	GpkWatch *watch = GPK_WATCH (data);
-	GladeXML *glade_xml;
+	GtkBuilder *builder;
 	GtkWidget *main_window;
 	GtkWidget *widget;
 	GtkListStore *list_store;
@@ -831,9 +818,19 @@ gpk_watch_menu_show_messages_cb (GtkMenuItem *item, gpointer data)
 	GtkTreeModel *model;
 	guint i;
 	GpkWatchCachedMessage *cached_message;
+	guint retval;
+	GError *error = NULL;
 
-	glade_xml = glade_xml_new (GPK_DATA "/gpk-repo.glade", NULL, NULL);
-	main_window = glade_xml_get_widget (glade_xml, "dialog_repo");
+	/* get UI */
+	builder = gtk_builder_new ();
+	retval = gtk_builder_add_from_file (builder, GPK_DATA "/gpk-repo.ui", &error);
+	if (error != NULL) {
+		egg_warning ("failed to load ui: %s", error->message);
+		g_error_free (error);
+		goto out_build;
+	}
+
+	main_window = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_repo"));
 	gtk_window_set_icon_name (GTK_WINDOW (main_window), GPK_ICON_SOFTWARE_LOG);
 	gtk_window_set_title (GTK_WINDOW (main_window), _("Package Manager Messages"));
 
@@ -843,19 +840,19 @@ gpk_watch_menu_show_messages_cb (GtkMenuItem *item, gpointer data)
 	/* Get the main window quit */
 	g_signal_connect_swapped (main_window, "delete_event", G_CALLBACK (gtk_main_quit), NULL);
 
-	widget = glade_xml_get_widget (glade_xml, "button_close");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_close"));
 	g_signal_connect_swapped (widget, "clicked", G_CALLBACK (gtk_main_quit), NULL);
-	widget = glade_xml_get_widget (glade_xml, "button_help");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_help"));
 	gtk_widget_hide (widget);
 
-	widget = glade_xml_get_widget (glade_xml, "checkbutton_detail");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "checkbutton_detail"));
 	gtk_widget_hide (widget);
 
 	/* create list stores */
 	list_store = gtk_list_store_new (GPK_WATCH_COLUMN_LAST, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 
 	/* create repo tree view */
-	widget = glade_xml_get_widget (glade_xml, "treeview_repo");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "treeview_repo"));
 	gtk_tree_view_set_model (GTK_TREE_VIEW (widget), GTK_TREE_MODEL (list_store));
 
 	/* column for text */
@@ -900,7 +897,7 @@ gpk_watch_menu_show_messages_cb (GtkMenuItem *item, gpointer data)
 	gtk_widget_show (main_window);
 
 	/* focus back to the close button */
-	widget = glade_xml_get_widget (glade_xml, "button_close");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_close"));
 	gtk_widget_grab_focus (widget);
 
 	/* wait */
@@ -911,14 +908,223 @@ gpk_watch_menu_show_messages_cb (GtkMenuItem *item, gpointer data)
 	g_ptr_array_foreach (watch->priv->cached_messages, (GFunc) gpk_watch_cached_message_free, NULL);
 	g_ptr_array_set_size (watch->priv->cached_messages, 0);
 
-	g_object_unref (glade_xml);
 	g_object_unref (list_store);
+out_build:
+	g_object_unref (builder);
 
 	/* refresh UI */
 	gpk_watch_refresh_icon (watch);
 	gpk_watch_refresh_tooltip (watch);
 }
 
+/**
+ * gpk_watch_get_role_text:
+ **/
+static gchar *
+gpk_watch_get_role_text (PkClient *client)
+{
+	const gchar *role_text;
+	gchar *text;
+	gchar *message;
+	PkRoleEnum role;
+	GError *error = NULL;
+	gboolean ret;
+
+	/* get role and text */
+	ret = pk_client_get_role (client, &role, &text, &error);
+	if (!ret) {
+		egg_warning ("failed to get role: %s", error->message);
+		g_error_free (error);
+		return NULL;
+	}
+
+	/* backup */
+	role_text = gpk_role_enum_to_localised_present (role);
+
+	if (!egg_strzero (text) && role != PK_ROLE_ENUM_UPDATE_PACKAGES)
+		message = g_strdup_printf ("%s: %s", role_text, text);
+	else
+		message = g_strdup_printf ("%s", role_text);
+	g_free (text);
+
+	return message;
+}
+
+/**
+ * gpk_watch_progress_changed_cb:
+ **/
+static void
+gpk_watch_progress_changed_cb (PkClient *client, guint percentage, guint subpercentage,
+				guint elapsed, guint remaining, GpkWatch *watch)
+{
+	gpk_modal_dialog_set_percentage (watch->priv->dialog, percentage);
+	gpk_modal_dialog_set_remaining (watch->priv->dialog, remaining);
+}
+
+/**
+ * gpk_watch_set_status:
+ **/
+static gboolean
+gpk_watch_set_status (GpkWatch *watch, PkStatusEnum status)
+{
+	/* do we force progress? */
+	if (status == PK_STATUS_ENUM_DOWNLOAD_REPOSITORY ||
+	    status == PK_STATUS_ENUM_DOWNLOAD_PACKAGELIST ||
+	    status == PK_STATUS_ENUM_DOWNLOAD_FILELIST ||
+	    status == PK_STATUS_ENUM_DOWNLOAD_CHANGELOG ||
+	    status == PK_STATUS_ENUM_DOWNLOAD_GROUP ||
+	    status == PK_STATUS_ENUM_DOWNLOAD_UPDATEINFO ||
+	    status == PK_STATUS_ENUM_REFRESH_CACHE) {
+		gpk_modal_dialog_setup (watch->priv->dialog, GPK_MODAL_DIALOG_PAGE_PROGRESS, 0);
+	}
+
+	/* set icon */
+	gpk_modal_dialog_set_image_status (watch->priv->dialog, status);
+
+	/* set label */
+	gpk_modal_dialog_set_title (watch->priv->dialog, gpk_status_enum_to_localised_text (status));
+
+	/* spin */
+	if (status == PK_STATUS_ENUM_WAIT)
+		gpk_modal_dialog_set_percentage (watch->priv->dialog, PK_CLIENT_PERCENTAGE_INVALID);
+
+	/* do visual stuff when finished */
+	if (status == PK_STATUS_ENUM_FINISHED) {
+		/* make insensitive */
+		gpk_modal_dialog_set_allow_cancel (watch->priv->dialog, FALSE);
+
+		/* stop spinning */
+		gpk_modal_dialog_set_percentage (watch->priv->dialog, 100);
+	}
+	return TRUE;
+}
+
+/**
+ * gpk_watch_status_changed_cb:
+ **/
+static void
+gpk_watch_status_changed_cb (PkClient *client, PkStatusEnum status, GpkWatch *watch)
+{
+	gpk_watch_set_status (watch, status);
+}
+
+/**
+ * gpk_watch_package_cb:
+ **/
+static void
+gpk_watch_package_cb (PkClient *client, const PkPackageObj *obj, GpkWatch *watch)
+{
+	gchar *text;
+	text = gpk_package_id_format_twoline (obj->id, obj->summary);
+	gpk_modal_dialog_set_message (watch->priv->dialog, text);
+	g_free (text);
+}
+
+/**
+ * gpk_watch_monitor_tid:
+ **/
+static gboolean
+gpk_watch_monitor_tid (GpkWatch *watch, const gchar *tid)
+{
+	PkStatusEnum status;
+	gboolean ret;
+	gboolean allow_cancel;
+	gchar *text;
+	gchar *package_id = NULL;
+	guint percentage;
+	guint subpercentage;
+	guint elapsed;
+	guint remaining;
+	GError *error = NULL;
+	PkRoleEnum role;
+
+	/* reset client */
+	ret = pk_client_reset (watch->priv->client_primary, &error);
+	if (!ret) {
+		egg_warning ("failed to reset client: %s", error->message);
+		g_error_free (error);
+		return FALSE;
+	}
+
+	ret = pk_client_set_tid (watch->priv->client_primary, tid, &error);
+	if (!ret) {
+		egg_warning ("could not set tid: %s", error->message);
+		g_error_free (error);
+		return FALSE;
+	}
+
+	/* fill in role */
+	text = gpk_watch_get_role_text (watch->priv->client_primary);
+	gpk_modal_dialog_set_title (watch->priv->dialog, text);
+	g_free (text);
+
+	/* coldplug */
+	ret = pk_client_get_status (watch->priv->client_primary, &status, NULL);
+	/* no such transaction? */
+	if (!ret) {
+		egg_warning ("could not get status");
+		return FALSE;
+	}
+
+	/* are we cancellable? */
+	pk_client_get_allow_cancel (watch->priv->client_primary, &allow_cancel, NULL);
+	gpk_modal_dialog_set_allow_cancel (watch->priv->dialog, allow_cancel);
+
+	/* coldplug */
+	ret = pk_client_get_progress (watch->priv->client_primary,
+				      &percentage, &subpercentage, &elapsed, &remaining, NULL);
+	if (ret) {
+		gpk_watch_progress_changed_cb (watch->priv->client_primary, percentage,
+						subpercentage, elapsed, remaining, watch);
+	} else {
+		egg_warning ("GetProgress failed");
+		gpk_watch_progress_changed_cb (watch->priv->client_primary,
+						PK_CLIENT_PERCENTAGE_INVALID,
+						PK_CLIENT_PERCENTAGE_INVALID, 0, 0, watch);
+	}
+
+	/* get the role */
+	ret = pk_client_get_role (watch->priv->client_primary, &role, NULL, &error);
+	if (!ret) {
+		egg_warning ("failed to get role: %s", error->message);
+		g_error_free (error);
+	}
+
+	/* setup the UI */
+	if (role == PK_ROLE_ENUM_SEARCH_NAME ||
+	    role == PK_ROLE_ENUM_SEARCH_GROUP ||
+	    role == PK_ROLE_ENUM_SEARCH_DETAILS ||
+	    role == PK_ROLE_ENUM_SEARCH_FILE ||
+	    role == PK_ROLE_ENUM_SEARCH_NAME ||
+	    role == PK_ROLE_ENUM_GET_UPDATES)
+		gpk_modal_dialog_setup (watch->priv->dialog, GPK_MODAL_DIALOG_PAGE_PROGRESS, 0);
+	else
+		gpk_modal_dialog_setup (watch->priv->dialog, GPK_MODAL_DIALOG_PAGE_PROGRESS, GPK_MODAL_DIALOG_PACKAGE_PADDING);
+
+	/* set the status */
+	gpk_watch_set_status (watch, status);
+
+	/* do the best we can, and get the last package */
+	ret = pk_client_get_package (watch->priv->client_primary, &package_id, NULL);
+	if (ret) {
+		PkPackageId *id;
+		PkPackageObj *obj;
+
+		id = pk_package_id_new_from_string (package_id);
+		if (id != NULL) {
+			obj = pk_package_obj_new (PK_INFO_ENUM_UNKNOWN, id, NULL);
+			egg_warning ("package_id=%s", package_id);
+			gpk_watch_package_cb (watch->priv->client_primary, obj, watch);
+			pk_package_obj_free (obj);
+		}
+		pk_package_id_free (id);
+	}
+
+	gpk_modal_dialog_present (watch->priv->dialog);
+
+	return TRUE;
+}
+
 /**
  * gpk_watch_menu_job_status_cb:
  **/
@@ -937,7 +1143,7 @@ gpk_watch_menu_job_status_cb (GtkMenuItem *item, GpkWatch *watch)
 	}
 
 	/* launch the UI */
-	gpk_client_monitor_tid (watch->priv->monitor, tid);
+	gpk_watch_monitor_tid (watch, tid);
 }
 
 /**
@@ -1004,7 +1210,9 @@ gpk_watch_menu_hide_restart_cb (GtkMenuItem *item, gpointer data)
 {
 	GpkWatch *watch = GPK_WATCH (data);
 	g_return_if_fail (GPK_IS_WATCH (watch));
-	gpk_smart_icon_set_icon_name (watch->priv->sicon, NULL);
+
+	/* hide */
+	gtk_status_icon_set_visible (watch->priv->status_icon, FALSE);
 }
 
 /**
@@ -1040,24 +1248,6 @@ gpk_watch_activate_status_cb (GtkStatusIcon *status_icon, GpkWatch *watch)
 	/* add jobs as drop down */
 	len = gpk_watch_populate_menu_with_jobs (watch, menu);
 
-	/* force a refresh if we are not updating or refreshing */
-	if (watch->priv->show_refresh_in_menu) {
-
-		/* Separator for HIG? */
-		if (len > 0) {
-			widget = gtk_separator_menu_item_new ();
-			gtk_menu_shell_append (GTK_MENU_SHELL (menu), widget);
-		}
-
-		/* TRANSLATORS: This is a right click menu item, and will refresh all the package lists */
-		widget = gtk_image_menu_item_new_with_mnemonic (_("_Refresh Software List"));
-		image = gtk_image_new_from_icon_name ("view-refresh", GTK_ICON_SIZE_MENU);
-		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (widget), image);
-		g_signal_connect (G_OBJECT (widget), "activate",
-				  G_CALLBACK (gpk_watch_menu_refresh_cache_cb), watch);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), widget);
-	}
-
 	/* any messages to show? */
 	len = watch->priv->cached_messages->len;
 	if (len > 0) {
@@ -1119,7 +1309,7 @@ pk_connection_changed_cb (PkConnection *pconnection, gboolean connected, GpkWatc
 		gpk_watch_refresh_icon (watch);
 		gpk_watch_refresh_tooltip (watch);
 	} else {
-		gpk_smart_icon_set_icon_name (watch->priv->sicon, NULL);
+		gtk_status_icon_set_visible (watch->priv->status_icon, FALSE);
 	}
 }
 
@@ -1317,6 +1507,59 @@ gpk_watch_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *
 	gpk_watch_set_proxies (watch);
 }
 
+/**
+ * gpk_watch_allow_cancel_cb:
+ **/
+static void
+gpk_watch_allow_cancel_cb (PkClient *client, gboolean allow_cancel, GpkWatch *watch)
+{
+	gpk_modal_dialog_set_allow_cancel (watch->priv->dialog, allow_cancel);
+}
+
+/**
+ * gpk_watch_finished_cb:
+ **/
+static void
+gpk_watch_finished_cb (PkClient *client, PkExitEnum exit_enum, guint runtime, GpkWatch *watch)
+{
+	g_return_if_fail (GPK_IS_WATCH (watch));
+
+	/* stop spinning */
+	gpk_modal_dialog_set_percentage (watch->priv->dialog, 100);
+
+	/* autoclose if success */
+	if (exit_enum == PK_EXIT_ENUM_SUCCESS) {
+		gpk_modal_dialog_close (watch->priv->dialog);
+	}
+}
+
+/**
+ * gpk_watch_button_close_cb:
+ **/
+static void
+gpk_watch_button_close_cb (GtkWidget *widget, GpkWatch *watch)
+{
+	/* close, don't abort */
+	gpk_modal_dialog_close (watch->priv->dialog);
+}
+
+/**
+ * gpk_watch_button_cancel_cb:
+ **/
+static void
+gpk_watch_button_cancel_cb (GtkWidget *widget, GpkWatch *watch)
+{
+	gboolean ret;
+	GError *error = NULL;
+
+	/* we might have a transaction running */
+	ret = pk_client_cancel (watch->priv->client_primary, &error);
+	if (!ret) {
+		egg_warning ("failed to cancel client: %s", error->message);
+		g_error_free (error);
+	}
+}
+
 /**
  * gpk_watch_init:
  * @watch: This class instance
@@ -1324,7 +1567,6 @@ gpk_watch_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *
 static void
 gpk_watch_init (GpkWatch *watch)
 {
-	GtkStatusIcon *status_icon;
 	PolKitAction *pk_action;
 	PolKitGnomeAction *restart_action;
 
@@ -1333,17 +1575,31 @@ gpk_watch_init (GpkWatch *watch)
 	watch->priv->notification_cached_messages = NULL;
 	watch->priv->restart = PK_RESTART_ENUM_NONE;
 
-	watch->priv->show_refresh_in_menu = TRUE;
 	watch->priv->gconf_client = gconf_client_get_default ();
 
-	watch->priv->sicon = gpk_smart_icon_new ();
+	watch->priv->status_icon = gtk_status_icon_new ();
 	watch->priv->set_proxy_timeout = 0;
-	watch->priv->gclient = gpk_client_new ();
 	watch->priv->cached_messages = g_ptr_array_new ();
 	watch->priv->restart_package_names = g_ptr_array_new ();
 
-	watch->priv->monitor = gpk_client_new ();
-	gpk_client_set_interaction (watch->priv->monitor, GPK_CLIENT_INTERACT_WARNING_PROGRESS);
+	watch->priv->client_primary = pk_client_new ();
+	g_signal_connect (watch->priv->client_primary, "finished",
+			  G_CALLBACK (gpk_watch_finished_cb), watch);
+	g_signal_connect (watch->priv->client_primary, "progress-changed",
+			  G_CALLBACK (gpk_watch_progress_changed_cb), watch);
+	g_signal_connect (watch->priv->client_primary, "status-changed",
+			  G_CALLBACK (gpk_watch_status_changed_cb), watch);
+	g_signal_connect (watch->priv->client_primary, "package",
+			  G_CALLBACK (gpk_watch_package_cb), watch);
+	g_signal_connect (watch->priv->client_primary, "allow-cancel",
+			  G_CALLBACK (gpk_watch_allow_cancel_cb), watch);
+
+	watch->priv->dialog = gpk_modal_dialog_new ();
+	gpk_modal_dialog_set_window_icon (watch->priv->dialog, "pk-package-installed");
+	g_signal_connect (watch->priv->dialog, "cancel",
+			  G_CALLBACK (gpk_watch_button_cancel_cb), watch);
+	g_signal_connect (watch->priv->dialog, "close",
+			  G_CALLBACK (gpk_watch_button_close_cb), watch);
 
 	/* we need to get ::locked */
 	watch->priv->control = pk_control_new ();
@@ -1354,10 +1610,9 @@ gpk_watch_init (GpkWatch *watch)
 	watch->priv->inhibit = gpk_inhibit_new ();
 
 	/* right click actions are common */
-	status_icon = GTK_STATUS_ICON (watch->priv->sicon);
-	g_signal_connect_object (G_OBJECT (status_icon),
+	g_signal_connect_object (G_OBJECT (watch->priv->status_icon),
 				 "popup_menu", G_CALLBACK (gpk_watch_popup_menu_cb), watch, 0);
-	g_signal_connect_object (G_OBJECT (status_icon),
+	g_signal_connect_object (G_OBJECT (watch->priv->status_icon),
 				 "activate", G_CALLBACK (gpk_watch_activate_status_cb), watch, 0);
 
 	watch->priv->tlist = pk_task_list_new ();
@@ -1366,7 +1621,7 @@ gpk_watch_init (GpkWatch *watch)
 	g_signal_connect (watch->priv->tlist, "status-changed",
 			  G_CALLBACK (gpk_watch_task_list_changed_cb), watch);
 	g_signal_connect (watch->priv->tlist, "finished",
-			  G_CALLBACK (gpk_watch_finished_cb), watch);
+			  G_CALLBACK (gpk_watch_task_list_finished_cb), watch);
 	g_signal_connect (watch->priv->tlist, "error-code",
 			  G_CALLBACK (gpk_watch_error_code_cb), watch);
 	g_signal_connect (watch->priv->tlist, "message",
@@ -1437,15 +1692,15 @@ gpk_watch_finalize (GObject *object)
 	g_ptr_array_free (watch->priv->restart_package_names, TRUE);
 
 	g_free (watch->priv->error_details);
-	g_object_unref (watch->priv->sicon);
+	g_object_unref (watch->priv->status_icon);
 	g_object_unref (watch->priv->inhibit);
 	g_object_unref (watch->priv->tlist);
 	g_object_unref (watch->priv->control);
-	g_object_unref (watch->priv->gclient);
 	g_object_unref (watch->priv->pconnection);
 	g_object_unref (watch->priv->gconf_client);
 	g_object_unref (watch->priv->restart_action);
-	g_object_unref (watch->priv->monitor);
+	g_object_unref (watch->priv->client_primary);
+	g_object_unref (watch->priv->dialog);
 
 	G_OBJECT_CLASS (gpk_watch_parent_class)->finalize (object);
 }
